/* Simple interface module for standardized calls of Motif widgets */
/* simplifies the parameter lists. All functions have fully */
/* type-checked prototypes with ANSI C and none with K&R C */
/* Lutz Prechelt, 1995/05/31 */

#include <Xm/Xm.h>
#define __USE_FIXED_PROTOTYPES__
#include <stdio.h>
#include <stdlib.h>

#ifdef __STDC__
#define _A_(l) l
#else
#define _A_(l) ()
#endif

/*************** Motif extensions ********************************/

/* artificial type-checking for resource-name <--> resource type */
typedef struct { String a; } XmIntResourceName;
typedef struct { String a; } XmFloatResourceName;
typedef struct { String a; } XmFuncResourceName;
typedef struct { String a; } XmStringResourceName;
typedef struct { String a; } XmWidgetResourceName;


/* type-checkable resource name constants */
extern XmIntResourceName
                  XmCcolumns,
                  XmCnumColumns,
                  XmCorientation,
                  XmCpacking,
                  XmCrows,
                  XmCwidth;
extern XmFuncResourceName
                  XmCactivateCallback,
                  XmCcancelCallback,
                  XmChelpCallback,
                  XmCokCallback;
extern XmStringResourceName
                  XmCvalue;
extern XmWidgetResourceName
                  XmCworkWindow;


/* ----- create a popup dialog window widget with an OK button */
Widget XmCreateErrorDialogOK _A_((Widget parent, String resourcename,
                              XmString message));

/* ----- create file selector dialog box */
Widget XmCreateFileSelectorDialog _A_((Widget w, String name));

/* ----- create a label widget with given label text */
Widget XmCreateLabelWidget _A_((String resourcename, Widget parent, 
                                XmString text));

/* ----- create a bare MainWindow widget */
Widget XmCreateMainWindowWidget _A_((String resourcename, Widget parent));

/* ----- create a pulldown menu with exactly three entries */
Widget XmCreatePulldownMenu3 _A_((String resourcename, Widget menubar, 
                            int menunumber,
                            XmString entry1, char key1,
                            XmString entry2, char key2,
                            XmString entry3, char key3,
                            void (*callback) (Widget, XtPointer, XtPointer)));

/* ----- create a PushButton widget with a given label */
Widget XmCreatePushButtonL _A_((String resourcename, Widget parent,
                            XmString label));

/* ----- create a RowColumn manager */
Widget XmCreateRowColumnManager _A_((String resourcename, Widget parent));

/* ----- create a preconfigured RowColumn manager */
Widget XmCreateRowColumnManagerOCP _A_((String resourcename, Widget parent,
                                        int orientation, int numColumns,
                                        Boolean equalize));

/* ----- create a scrolled text widget without any resources set */
Widget XmCreateScrolledTextWidget _A_((Widget parent, String resourcename));

/* ----- create a stand alone scrolled text widget of lines x columns */
Widget XmCreateScrolledTextWindow _A_((String resourcename, Widget parent,
                                      int nrOfLines, int nrOfColumns));

/* ----- create empty TextField widget */
Widget XmCreateTextFieldWidget _A_((String resourcename, Widget parent));

/* ----- create TextField widget with given width and initial string */
Widget XmCreateTextFieldWidgetW _A_((String resourcename, Widget parent,
                                     int width, String value));

/* ----- create MenuBar widget with exactly one entry */
Widget XmCreateTrivialMenuBar _A_((Widget parent, String resourcename,
                                   XmString menuname, char key));

/* ----- install callback function with type-checked resource name */
void XtAddCallbackF _A_((Widget w, XmFuncResourceName r, 
                     void (*f) (Widget, XtPointer, XtPointer),
                     XtPointer client_data));

/* ----- read a value of a resource from a widget */
void XtGetIntValue _A_((Widget w, XmStringResourceName r, int *value));
void XtGetFloatValue _A_((Widget w, XmFloatResourceName r, double *value));
void XtGetStringValue _A_((Widget w, XmStringResourceName r, String *value));
void XtGetWidgetValue _A_((Widget w, XmStringResourceName r, Widget *value));

/* ----- change the value of a resource for a widget */
void XtSetIntValue _A_((Widget w, XmIntResourceName r, int value));
void XtSetFloatValue _A_((Widget w, XmFloatResourceName r, double value));
void XtSetStringValue _A_((Widget w, XmStringResourceName r, String value));
void XtSetWidgetValue _A_((Widget w, XmWidgetResourceName r, Widget value));


/*************** other extensions ********************************/

/* ----- fallback resources ----- */
extern String fallbacks[];

/* ----- convert floating point number into String of given form */
String ftoa _A_((double x, int width, int precision));

/* ----- global Initialization (protocol stamp) */
void globalInitialize _A_((String programname));

/* ----- callback function: store selected filename from fileselector box */
void keepSelectedFile _A_((Widget w, XtPointer client_data,
                           XtPointer callback_data));

/* ----- print error message along with 2x2 matrix in given format */
void matrixErrorMessage _A_((String message, double matrixcoefficients[4],
                             int width, int precision));

/* ----- read file and return all its contents in one newly allocated string */
String readWholeFile _A_((String filename));

/* ----- retrieve filename stored by keepSelectedFile() */
String selectedFile _A_(());


